#!/usr/bin/env python

#-----------------------------------------------------------------------
# database.py
# Author: Bob Dondero
#-----------------------------------------------------------------------

import os
import sqlalchemy
import dotenv

#-----------------------------------------------------------------------

dotenv.load_dotenv()
_database_url = os.getenv('DATABASE_URL', 'sqlite:///penny.sqlite')
_database_url = _database_url.replace('postgres://', 'postgresql://')

#-----------------------------------------------------------------------

Base = sqlalchemy.orm.declarative_base()

class Book (Base):
    __tablename__ = 'books'
    isbn = sqlalchemy.Column(sqlalchemy.String, primary_key=True)
    author = sqlalchemy.Column(sqlalchemy.String)
    title = sqlalchemy.Column(sqlalchemy.String)

class User (Base):
    __tablename__ = 'users'
    username = sqlalchemy.Column(sqlalchemy.String, primary_key=True)
    password = sqlalchemy.Column(sqlalchemy.String)

class AuthorizedUser (Base):
    __tablename__ = 'authorizedusers'
    username = sqlalchemy.Column(sqlalchemy.String, primary_key=True)

_engine = sqlalchemy.create_engine(_database_url)

#-----------------------------------------------------------------------

def get_books():

    books = []

    with sqlalchemy.orm.Session(_engine) as session:
        query = session.query(Book)
        table = query.all()
        for row in table:
            book = {'isbn': row.isbn, 'author': row.author,
                'title': row.title}
            books.append(book)

    return books

#-----------------------------------------------------------------------

def add_book(isbn, author, title):

    with sqlalchemy.orm.Session(_engine) as session:
        row = Book(isbn=isbn, author=author, title=title)
        session.add(row)
        try:
            session.commit()
            return True
        except sqlalchemy.exc.IntegrityError:
            return False

#-----------------------------------------------------------------------

def delete_book(isbn):

    with sqlalchemy.orm.Session(_engine) as session:
        session.query(Book).filter(Book.isbn==isbn).delete()
        session.commit()

#-----------------------------------------------------------------------

def get_password(username):

    with sqlalchemy.orm.Session(_engine) as session:
        query = session.query(User).filter(User.username==username)
        try:
            row = query.one()
            return row.password
        except sqlalchemy.exc.NoResultFound:
            return None

#-----------------------------------------------------------------------

def is_authorized(username):

    with sqlalchemy.orm.Session(_engine) as session:
        query = session.query(AuthorizedUser) \
            .filter(AuthorizedUser.username==username)
        try:
            query.one()
            return True
        except sqlalchemy.exc.NoResultFound:
            return False

#-----------------------------------------------------------------------

def add_user(username, password):

    with sqlalchemy.orm.Session(_engine) as session:
        row = User(username=username, password=password)
        session.add(row)
        try:
            session.commit()
            return True
        except sqlalchemy.exc.IntegrityError:
            return False

#-----------------------------------------------------------------------

# For testing:

def _write_books(books):
    for book in books:
        print('%s | %s | %s' % (book['isbn'], book['author'],
            book['title']))

def _test():
    print('----------------------------------')
    print('Testing get_books()')
    print('----------------------------------')
    print()
    books = get_books()
    _write_books(books)
    print()

    print('----------------------------------')
    print('Testing add_book()')
    print('----------------------------------')
    print()
    successful = add_book('456', 'Kernighan', 'New Book')
    if successful:
        print('Add was successful')
        print()
        books = get_books()
        _write_books(books)
        print()
    else:
        print('Add was unsuccessful')
        print()
        _write_books(books)
        print()
    successful = add_book('456', 'Kernighan', 'New Book')
    if successful:
        print('Add was successful')
        print()
        books = get_books()
        _write_books(books)
        print()
    else:
        print('Add was unsuccessful')
        print()
        _write_books(books)
        print()

    print('----------------------------------')
    print('Testing delete_book()')
    print('----------------------------------')
    print()
    delete_book('456')
    books = get_books()
    _write_books(books)
    print()
    delete_book('456')
    books = get_books()
    _write_books(books)
    print()

    print('----------------------------------')
    print('Testing get_password()')
    print('----------------------------------')
    print()
    password = get_password('rdondero')
    print(password)
    password = get_password('rdondero2')
    print(password)
    print()

if __name__ == '__main__':
    _test()
